# -*- coding: utf-8 -*-
import scrapy, re, json
from elFinanciero.items import NoticiasItem
from collections import OrderedDict
from datetime import datetime, date, timedelta, tzinfo

"""
MEDIO:
El Financiero, CDMX

DESCARGA HACIA ATRÁS:
Este crawler no descarga las noticias de un día específico, sino que descarga las todas las noticias desde
la fecha más reciente hasta la fecha indicada con los parámetros 'year', 'month', 'day'  

USO:
scrapy crawl noticias --nolog -s filename=2018-02-06.json -a year=2018 -a month=2 -a day=6
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')
TIME = re.compile(r'\d{1,2}:\d{2} ?[ap]m')
PUB = re.compile(r'"publishedAt":.*?,')


class ImportantData(scrapy.Item):
    section = scrapy.Field()
    url1 = scrapy.Field()
    url2 = scrapy.Field()
    page = scrapy.Field()
    res = scrapy.Field()


class SectionData(scrapy.Item):
    section = scrapy.Field()


class UTC(tzinfo):
	"""clase para el 'time zone' (zona horaria)"""

	def utcoffset(self, dt):
		# zona horaria para aguascalientes (centro de méxico): utc-6
		return timedelta(hours=-6)

	def tzname(self, dt):
		# nombre de la zona horaria
		return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"
    newsSet = set()

    def start_requests(self):
        self.tz = UTC()
        # year = getattr(self, "year", None)
        # month = getattr(self, "month", None)
        # day = getattr(self, "day", None)
        # self.currentDate = date(int(year), int(month), int(day))
        self.currentDate = datetime.now().date()
        # self.currentDate = date(2018, 2, 23)

        self.date_parser = {'enero': 1,      'febrero': 2,  'marzo': 3,      'abril': 4,
                            'mayo': 5,       'junio': 6,    'julio': 7,      'agosto': 8,
                            'septiembre': 9, 'octubre': 10, 'noviembre': 11, 'diciembre': 12}

        sectionList = ["economia", "empresas", "nacional", "culturas",
                       "deportes", "mundo", "bajio", "tech", "ciencia"]

        self.baseURL = "http://www.elfinanciero.com.mx/"

        """
        Ejemplo de URL para las noticias de días anteriores para la sección Economía:
        http://api.elfinanciero.com.mx/public/search/typed/?_format=json&json={%22search%22:%22*%22,%22categoriesslug%22:%22economia%22}&type=page&page=2&size=10
        """
        self.uri_base = "http://api.elfinanciero.com.mx/public/search/typed/?_format=json&json={%22search%22:%22*%22,%22categoriesslug%22:%22"
        self.uri_page = "%22}&type=page&page="
        self.uri_complement = "&size=10"

        for s in sectionList:
            yield scrapy.Request(url=self.baseURL + s, callback=self.parse)


    def parse(self, response):
        searchData = ImportantData()

        CONTINUE_SEARCHING = True
        section = response.url[response.url.rfind("/") + 1:]

        for link in response.css('div.is-multiline').css('div.column-box').xpath('./a/@href').extract():
            if link.find("/") == 0:
                link = link[1:]
            yield scrapy.Request(url=self.baseURL + link, callback=self.parse_item)

        newsLinkList = response.xpath('//div[@class="column feed"]/a/@href').extract()
        newsDateList = response.xpath('//div[@class="column feed"]').css('p.date-time::text').extract()
        postDict = OrderedDict(zip(newsLinkList, newsDateList))

        for uri in postDict.keys():
            dt = postDict[uri]
            res = TIME.match(dt)
            if res:
                postDate = datetime.now().date()
            else:
                postDate = datetime.strptime(dt, "%d/%m/%Y").date()

            if postDate >= self.currentDate:
                if uri.find("/") == 0:
                    uri = uri[1:]
                yield scrapy.Request(url=self.baseURL + uri, callback=self.parse_item)

            else:
                CONTINUE_SEARCHING = False
                break

        if CONTINUE_SEARCHING:
            page = 2
            url = self.uri_base + section + self.uri_page + str(page) + self.uri_complement
            searchData['section'] = section
            searchData['page'] = page
            request = scrapy.Request(url=url, callback=self.continue_searching, dont_filter=True)
            request.meta['item'] = searchData
            yield request


    def continue_searching(self, response):
        CONTINUE_SEARCHING = True
        searchData = response.meta['item']
        REG_EXPR = re.compile(r'"' + re.escape(searchData['section']) + r'\\/.*?"')

        lList = REG_EXPR.findall(response.body)
        pList = PUB.findall(response.body)
        linkList = [l.replace("\\", '').replace('"', '') for l in lList]
        isodateList = [d[d.find(":")+1:].replace('"', '').replace(",", '') for d in pList]
        postDict = OrderedDict(zip(linkList, [iso[:iso.find("T")] for iso in isodateList]))

        for uri in postDict.keys():
            dt = postDict[uri]
            res = TIME.match(dt)
            if res:
                postDate = datetime.now().date()
            else:
                postDate = datetime.strptime(dt, "%Y-%m-%d").date()

            if postDate >= self.currentDate:
                if uri.find("/") == 0:
                    uri = uri[1:]
                yield scrapy.Request(url=self.baseURL + uri, callback=self.parse_item)

            else:
                CONTINUE_SEARCHING = False
                break

        if CONTINUE_SEARCHING:
            searchData['page'] += 1
            url = self.uri_base + searchData['section'] + self.uri_page + str(searchData['page']) + self.uri_complement
            request = scrapy.Request(url=url, callback=self.continue_searching)
            request.meta['item'] = searchData
            yield request


    def parse_item(self, response):
        if not response.url in self.newsSet:
            self.newsSet.add(response.url)
            item = NoticiasItem()
            text = ''

            res = remove_tags(response.xpath('//script[@type="application/ld+json"]').extract_first())
            resDict = json.loads(res)
            dt = resDict['datePublished']
            d, t = dt.split()
            d = map(int, d.split("-"))
            t = map(int, t.split(":"))
            dat = date(d[0], d[1], d[2])

            if dat >= self.currentDate:
                item['date'] = datetime(d[0], d[1], d[2], t[0], t[1], t[2], tzinfo=self.tz).isoformat("T")

                item['title'] = remove_tags(response.css('div.column').css('div.column').css('h1').extract_first()).strip()

                topic = response.xpath('//div[@class="section-line"]').extract_first()
                if topic is not None:
                    item['topic'] = remove_tags(topic)
                else:
                    item['topic'] = None

                author = response.xpath('//div[@class="note-author"]/a').extract_first()
                if author is not None:
                    item['author'] = remove_tags(author)

                for p in response.css('div.content').css('p').extract():
                    text += remove_tags(p) + '\n'

                # result = LOC_RE.search(text)
                # if result:
                #     m = result.group(0)
                #     location = G_RE.sub('', m).strip()
                #     if len(location) <= 35:
                #         item['location'] = location
                #         text = text[text.find(m)+len(m):]

                # text = EM_RE.sub('', text)
                # text = TW_RE.sub('', text)
                # text = TW2_RE.sub('', text)
                # text = TAG2_RE.sub("\n", text)
                # text = TAG3_RE.sub('', text)
                item['text'] = text.strip()

                item['url'] = response.url

                yield item
