# -*- coding: utf-8 -*-
import scrapy, re
from solDelCentro.items import NoticiasItem
from datetime import datetime, date, timedelta, tzinfo

"""
MEDIO:
El Sol del Centro, Aguascalientes

DESCARGA HACIA ATRÁS:
Este crawler no descarga las noticias de un día específico, sino que descarga las todas las noticias desde
la fecha más reciente hasta la fecha indicada con los parámetros 'year', 'month', 'day'  

USO:
scrapy crawl noticias --nolog -s filename=2018-02-03.json -a year=2018 -a month=2 -a day=3
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')


class ImportantData(scrapy.Item):
    section = scrapy.Field()
    url1 = scrapy.Field()
    url2 = scrapy.Field()
    page = scrapy.Field()
    res = scrapy.Field()


class SectionData(scrapy.Item):
    section = scrapy.Field()


class UTC(tzinfo):
	"""clase para el 'time zone' (zona horaria)"""

	def utcoffset(self, dt):
		# zona horaria para aguascalientes (centro de méxico): utc-6
		return timedelta(hours=-6)

	def tzname(self, dt):
		# nombre de la zona horaria
		return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)
        self.currentDate = date(int(year), int(month), int(day))

        self.date_parser = {'enero': 1,      'febrero': 2,  'marzo': 3,      'abril': 4,
                            'mayo': 5,       'junio': 6,    'julio': 7,      'agosto': 8,
                            'septiembre': 9, 'octubre': 10, 'noviembre': 11, 'diciembre': 12}

        self.link_parser = {"local_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=local&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "local_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=local&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Flocal",
                            "policia_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=policia&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "policia_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=policia&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fpolicia",
                            "mexico_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=mexico&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "mexico_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=mexico&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fmexico",
                            "republica_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=republica&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "republica_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=republica&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Frepublica",
                            "mundo_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=mundo&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "mundo_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=mundo&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fmundo",
                            "finanzas_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=finanzas&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "finanzas_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=finanzas&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Ffinanzas",
                            "deportes_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=c070e82ff5&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=deportes&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "deportes_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=deportes&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fdeportes",
                            "espectaculos_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=post&post_format=&category=espectaculos&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "espectaculos_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=espectaculos&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fespectaculos",
                            "columna_1": "https://www.elsoldelcentro.com.mx/wp-admin/admin-ajax.php?action=alm_query_posts&query_type=standard&nonce=9598a706d3&repeater=default&theme_repeater=null&cta=&comments=&post_type%5B%5D=columnas&post_format=&category=&category__not_in=&tag=&tag__not_in=&taxonomy=&taxonomy_terms=&taxonomy_operator=&taxonomy_relation=&meta_key=&meta_value=&meta_compare=&meta_relation=&meta_type=&author=&year=&month=&day=&post_status=&order=DESC&orderby=date&post__in=&post__not_in=&exclude=&search=&custom_args=&posts_per_page=5&page=",
                            "columna_2": "&offset=10&preloaded=false&seo_start_page=1&paging=false&previous_post=false&previous_post_id=&previous_post_taxonomy=&lang=&slug=columnas&canonical_url=https%3A%2F%2Fwww.elsoldelcentro.com.mx%2Fcolumna"}

        sectionList = ["local", "policia", "mexico", "republica", "mundo",
                       "finanzas", "deportes", "espectaculos", "columna"]

        baseURL = "https://www.elsoldelcentro.com.mx/"

        for s in sectionList:
            yield scrapy.Request(url=baseURL + s, callback=self.parse)


    def parse(self, response):
        searchData = ImportantData()
        newsSection = SectionData()

        CONTINUE_SEARCHING = True
        section = response.url[response.url.rfind("/") + 1:]
        newsSection['section'] = section

        for post in response.xpath('//div[@class="col-xs-12 col-md-12 col-lg-8 col-izq"]').css('article'):
            res = post.css('time::attr(datetime)').extract_first()

            if res is not None:
                d = res.split()[0]
                d = map(int, d.split("-"))
                postDate = date(d[0], d[1], d[2])

                if postDate >= self.currentDate:
                    link = post.css('div.tit-nota').xpath('./h2/a/@href').extract_first()
                    req = scrapy.Request(url=link, callback=self.parse_item)
                    req.meta['item'] = newsSection
                    yield req

                else:
                    CONTINUE_SEARCHING = False
                    break

        if CONTINUE_SEARCHING:
            url1 = self.link_parser[section + "_1"]
            url2 = self.link_parser[section + "_2"]
            page = 0
            url = url1 + str(page) + url2
            searchData['section'] = section
            searchData['url1'] = url1
            searchData['url2'] = url2
            searchData['page'] = page
            searchData['res'] = response.url + "/"
            request = scrapy.Request(url=url, callback=self.continue_searching, dont_filter=True)
            request.meta['item'] = searchData
            yield request


    def continue_searching(self, response):
        newsSection = SectionData()
        searchData = response.meta['item']

        CONTINUE_SEARCHING = True
        res = searchData['res']
        newsSection['section'] = searchData['section']
        dateList = response.xpath('//p[@class=\'\\"entry-meta\\"\']').extract()
        titleList = response.xpath('//li/h3/a/@href').extract()

        if len(dateList) > 0:
            for i in range(0, len(dateList)):
                d = dateList[i]
                d = remove_tags(d).replace(",", '').split()
                postDate = date(int(d[2]), self.date_parser[d[0].lower()], int(d[1]))

                if postDate >= self.currentDate:
                    l = titleList[i]
                    l = l[l.rfind("/")+1:-2]
                    link = res + l
                    req = scrapy.Request(url=link, callback=self.parse_item)
                    req.meta['item'] = newsSection
                    yield req

                else:
                    CONTINUE_SEARCHING = False
                    break

        else:
            CONTINUE_SEARCHING = False


        if CONTINUE_SEARCHING:
            url1 = searchData['url1']
            url2 = searchData['url2']
            searchData['page'] += 1
            page = searchData['page']
            request = scrapy.Request(url=url1 + str(page) + url2, callback=self.continue_searching)
            request.meta['item'] = searchData
            yield request


    # def parse_page(self, response):
    #     print "URL: " + response.url


    def parse_item(self, response):
        item = NoticiasItem()
        newsSection = response.meta['item']
        text = ''

        res = response.css('div.info-creditos-notas').css('time::attr(datetime)').extract_first()
        d,t = res.split()
        d = map(int, d.split("-"))
        t = map(int, t.split(":"))
        dat = datetime(d[0], d[1], d[2], t[0], t[1], 0, tzinfo=self.tz).isoformat("T")
        item['date'] = dat

        item['title'] = remove_tags(response.css('div.col-xs-12').xpath('./h1').extract_first()).strip()

        topic = response.xpath('//div[@class="info-creditos-notas"]/ul/li[@class="seccion"]/a/text()').extract_first()
        if topic is None:
            topic = newsSection['section'].capitalize()
        item['topic'] = topic

        for p in response.css('div.col-xs-12').css('p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
