# -*- coding: utf-8 -*-
import scrapy, re
from elIndependiente.items import NoticiasItem

"""
MEDIO:
El Independiente, Michoacán
USO:
scrapy crawl noticias --nolog -s filename=2018-02-01.json -a year=2018 -a month=2 -a day=1
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "http://www.el-independiente.com.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.xpath('//div[@class="pagination"]/a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-1].strip('/')
            pages = int(pagination[pagination.rfind('/') + 1:])

            for page in range(1, pages):
                yield scrapy.Request(url=self.baseURL + '/page/' + str(page + 1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//div[@class="main-col"]/div/div[@class="bp-entry"]/div/h2/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//meta[@itemprop="datePublished"]/@content').extract_first()
        item['title'] = remove_tags(response.xpath('//h1[@class="post-tile entry-title"]').extract_first()).strip()

        topic = response.xpath('//div[@class="breadcrumbs-plus"]/span/a/span/text()').extract()[-1]
        item['topic'] = topic

        paragraph = response.xpath('//div[@class="pf-content"]/p').extract()
        paragraph.extend(response.xpath('//div[@class="pf-content"]/ul/li').extract())
        for p in paragraph:
            p = remove_tags(p)
            if p != "Borrar":
                text += p + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
