# -*- coding: utf-8 -*-
import scrapy, re
from elValle.items import NoticiasItem

"""
MEDIO:
El Valle, Edo. Méx.
USO:
scrapy crawl noticias --nolog -s filename=2018-02-01.json -a year=2018 -a month=2 -a day=1
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
G_RE = re.compile(r' ?- ?')
LOC2_RE = re.compile(r'\n.+?, ?.+? ?\.? ?- ?')
G2_RE = re.compile(r' ?\.? ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "http://elvalle.com.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        nextPage = response.xpath('//a[@class="next page-numbers"]/@href').extract_first()
        if nextPage is not None:
            yield scrapy.Request(url=nextPage, callback=self.parse)


    def parse_page(self, response):
        for link in response.xpath('//ul[@class="bk-blog-content clearfix"]/li/div/div/h4/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//meta[@itemprop="datePublished"]/@content').extract_first()
        item['title'] = remove_tags(response.xpath('//h1[@itemprop="headline"]').extract_first()).strip()

        item['topic'] = response.xpath('//div[@class="s-post-header container"]/div[@class="meta"]/div[@class="post-category"]/a/text()').extract_first()

        for p in response.xpath('//div[@class="article-content clearfix"]/p').extract():
            p = remove_tags(p)
            p = p.replace(u'\u2013', "-")
            p = p.replace(u'\u2014', "-")
            text += p + "\n"

        text = "\n" + text
        result = LOC_RE.search(text)
        if result:
            m = result.group(0)
            location = G_RE.sub('', m).strip()
            if len(location) <= 35:
                item['location'] = location
                text = text[text.find(m)+len(m):]
        else:
            result = LOC2_RE.search(text)
            if result:
                m = result.group(0)
                location = G2_RE.sub('', m).strip()
                if len(location) <= 35:
                    item['location'] = location
                    text = text[text.find(m) + len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
