# -*- coding: utf-8 -*-
import scrapy, re
from heraldoAgs.items import NoticiasItem

"""
MEDIO:
El Heraldo, Aguascalientes
USO:
scrapy crawl noticias --nolog -s filename=2018-01-05.json -a year=2018 -a month=1 -a day=5
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

LOC_RE = re.compile(r'.+?(\d{1,2}-?[a-zA-Z]+)?\s?\.-\s?')
DAT_RE = re.compile(r'\s?(\d{1,2}-?[a-zA-Z]+)?\s?\.-\s?')


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "http://www.heraldo.mx/" + year + "/" + month + "/" + day

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        lastPage = response.xpath('//*[@class="page-nav td-pb-padding-side"]/a[@class="last"]/text()').extract_first()
        if lastPage is not None and lastPage != '':
            lastPage = int(lastPage)
            for page in range(1,lastPage):
                yield scrapy.Request(url=self.baseURL + "/page/" + str(page+1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//*[@class="td-block-row"]').css('h3').css('a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//time[@class="entry-date updated td-module-date"]/@datetime').extract_first()
        item['title'] = remove_tags(response.xpath('//header[@class="td-post-title"]/h1').extract_first())
        item['topic'] = response.xpath('//ul[@class="td-category"]/li/a/text()').extract_first()

        author = response.xpath('//pre[@style="text-align: justify;"]/text()').extract_first()
        if author is not None and author != '':
            item['author'] = " ".join(author.split("\n")[0].split())

        bodyText = response.xpath('//*[@class="td-post-content"]/p').extract()
        for i in range(0, len(bodyText)):
            p = remove_tags(bodyText[i])
            if i <= 1:
                p = p.lstrip()
                result = LOC_RE.match(p)
                if result:
                    item['location'] = DAT_RE.sub('', result.group(0))
                    p = LOC_RE.sub('', p)

            text += p + "\n"
        item['text'] = text

        item['url'] = response.url

        yield item
