# -*- coding: utf-8 -*-
import scrapy, re
from laJornadaVer.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo

"""
MEDIO:
La Jornada de Veracruz, Ver.
USO:
scrapy crawl noticias --nolog -s filename=2017-03-22.json -a year=2017 -a month=3 -a day=22
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """clase para el 'time zone' (zona horaria)"""

    def utcoffset(self, dt):
        # zona horaria para veracruz (centro de mexico): utc-6
        return timedelta(hours=-6)

    def tzname(self, dt):
        # nombre de la zona horaria
        return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)
        day = getattr(self, 'day', None)
        self.date = datetime(int(year), int(month), int(day), tzinfo=tz).isoformat('T')

        self.baseURL = 'http://www.jornadaveracruz.com.mx/'
        self.builtURL= self.baseURL+'Archive.aspx?date='+day.zfill(2)+'/'+month.zfill(2)+'/'+year

        yield scrapy.Request(url=self.builtURL, callback=self.parse)


    def parse(self, response):
        paths = ['//*[@class="article-header"]/h2/a/@href',
                 '//ul[@class="article-array content-category"]/li/a/@href']

        for path in paths:
            links = response.xpath(path).extract()
            if len(links) > 0:
                for link in links:
                    yield scrapy.Request(url=self.baseURL+link, callback=self.parse_item)

        # for link in response.xpath('//*[@class="text_block_200"]/h2/a/@href').extract():
        # 	yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_page)


    def parse_page(self, response):
        paths = ['//*[@id="article_list"]/h2/a/@href',
                 '//*[@id="article_list"]/h3/a/@href']

        for path in paths:
            for link in response.xpath(path).extract():
                if not link in self.link_list:
                    yield scrapy.Request(url=self.baseURL+'/'+link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''
        item['date'] = self.date

        title = response.xpath('//h2[@class="article-title"]/text()').extract_first()
        title = title.replace('\r','')
        title = title.replace('\n','')
        title = title.lstrip(' ')
        title = title.rstrip(' ')
        item['title'] = title

        topic = response.xpath('//*[@class="content-article-title"]/h2/text()').extract_first()
        topic = topic.replace('\r','')
        topic = topic.replace('\n','')
        topic = topic.lstrip(' ')
        topic = topic.rstrip(' ')
        item['topic'] = topic
        # item['author'] = response.xpath('//*[@class="right-side"]/div/a[@rel="author"]/text()').extract_first()

        paragraph = response.xpath('//*[@class="shortcode-content"]/p/text()').extract()
        if len(paragraph) > 0:
            for p in paragraph:
                text += paragraph[3]
        else:
            for p in response.xpath('//*[@class="shortcode-content"]').extract():
                text += remove_tags(p) + '\n'

        item['text'] = text
        item['url'] = response.url
        # print item['title']
        yield item

