# -*- coding: utf-8 -*-
import scrapy, re
from noticiasBahia.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo

"""
MEDIO:
Noticias de la Bahía, Nayarit
USO:
scrapy crawl noticias --nolog -s filename=2018-02-01.json -a year=2018 -a month=2 -a day=1
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')


class UTC(tzinfo):
    """clase para el 'time zone' (zona horaria)"""

    def utcoffset(self, dt):
        # zona horaria para nayarit: utc-7
        return timedelta(hours=-7)

    def tzname(self, dt):
        # nombre de la zona horaria
        return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        self.year = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day = getattr(self, "day", None)
        self.date_parser = {"enero": 1,      "febrero": 2,  "marzo": 3,      "abril": 4,
                            "mayo": 5,       "junio": 6,    "julio": 7,      "agosto": 8,
                            "septiembre": 9, "octubre": 10, "noviembre": 11, "diciembre": 12}

        self.baseURL = "https://noticiasdelabahia.com/" + self.year + "/" + self.month.zfill(2) + "/" + self.day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        nextPage = response.xpath('//div[@class="pagination clearfix"]/a[@class="older-posts"]/@href').extract_first()
        if nextPage is not None:
            yield scrapy.Request(url=nextPage, callback=self.parse)


    def parse_page(self, response):
        for link in response.xpath('//div[@class="row default-layout"]/article/h2/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        d = response.xpath('//span[@class="date"]').extract_first()
        d = remove_tags(d).strip().replace(",", '').split()
        try:
            dat = datetime(int(d[2]), self.date_parser[d[1].lower()], int(d[0]), tzinfo=self.tz).isoformat("T")
        except:
            dat = datetime(int(self.year), int(self.month), int(self.day), tzinfo=self.tz).isoformat("T")
        item['date'] = dat

        item['title'] = remove_tags(response.xpath('//h1[@class="title"]').extract_first()).strip()

        topic = response.xpath('//div[@class="category-list"]/ul/li/a').extract_first()
        if topic is not None:
            item['topic'] = remove_tags(topic).strip()

        author = response.xpath('//span[@class="author"]').extract_first()
        if author is not None:
            item['author'] = remove_tags(author).strip()

        for p in response.xpath('//div[@class="post-entry"]/p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
