# -*- coding: utf-8 -*-
import scrapy, re
from notirivas.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo

"""
MEDIO:
Notirivas, Yucatán
USO:
scrapy crawl noticias --nolog -s filename=2017-03-22.json -a year=2017 -a month=3 -a day=22
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
    """clase para el 'time zone' (zona horaria)"""

    def utcoffset(self, dt):
        # zona horaria para yucatan (centro de mexico): utc-6
        return timedelta(hours=-6)

    def tzname(self, dt):
        # nombre de la zona horaria
        return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        tz = UTC()
        year = getattr(self, 'year', None)
        month = getattr(self, 'month', None)
        day = getattr(self, 'day', None)
        self.date = datetime(int(year), int(month), int(day), tzinfo=tz).isoformat('T')

        self.baseURL = "http://gruporivas.com.mx/notirivas/" + year + "/" + month + "/" +day

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.xpath('//*[@class="bdaia-pagination"]/span[@class="pages"]/text()').extract()
        if len(pagination) > 0:
            pagination = pagination[0]
            pages = int(pagination[pagination.rfind(' ')+1:])

            for page in range(1, pages):
                yield scrapy.Request(url=self.baseURL+"/page/"+str(page+1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//article/header/h2/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''
        item['date'] = self.date
        item['title'] = response.xpath('//*[@class="bdaia-post-title"]/h1/span/text()').extract_first()
        item['topic'] = response.xpath('//*[@class="bdaia-category"]/a/text()').extract_first()

        content = response.xpath('//*[@class="bdaia-post-content"]/p/text()').extract()
        if ( len(content) == 0 ):
            content = response.xpath('//*[@class="bdaia-post-content"]/div/div/p/text()').extract()
            if ( len(content) == 0 ):
                content = response.xpath('//*[@class="bdaia-post-content"]/p/span/text()').extract()

        for paragraph in content:
            text += remove_tags(paragraph) + '\n'
        item['text'] = text
        item['url'] = response.url

        # print item['title']
        yield item

