# -*- coding: utf-8 -*-
import scrapy, re
from opinionPuebla.items import NoticiasItem
from datetime import datetime, date, timedelta, tzinfo

"""
MEDIO:
La Opinión de Puebla
USO:
scrapy crawl noticias --nolog -s filename=2018-02-04.json -a year=2018 -a month=2 -a day=4
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')


class UTC(tzinfo):
	"""clase para el 'time zone' (zona horaria)"""

	def utcoffset(self, dt):
		# zona horaria para puebla: utc-6
		return timedelta(hours=-6)

	def tzname(self, dt):
		# nombre de la zona horaria
		return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        self.year = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day = getattr(self, "day", None)

        self.date_parser = {"enero": 1,      "febrero": 2,  "marzo": 3,      "abril": 4,
                            "mayo": 5,       "junio": 6,    "julio": 7,      "agosto": 8,
                            "septiembre": 9, "octubre": 10, "noviembre": 11, "diciembre": 12}

        self.baseURL = "http://www.laopinionpuebla.com/" + self.year + "/" + self.month.zfill(2) + "/" + self.day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.xpath('//ul[@class="page-numbers"]/li/a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-1]
            pages = int(pagination[pagination.rfind("=") + 1:])

            for page in range(1, pages):
                yield scrapy.Request(url=self.baseURL + '/page/' + str(page + 1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//div[@id="content"]').css('h2').xpath('./a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''
        CHANGE_HOUR = False

        try:
            res = response.xpath('//ul[@class="post-meta post"]/li[@class="date"]').extract_first()
            res = remove_tags(res)
            if res.rfind("pm") > -1: CHANGE_HOUR = True
            res = res.replace(" am", '').replace(" pm", '').replace(",", '').split()
            t = map(int, res[-1].split(":"))
            if CHANGE_HOUR: t[0] += 12
            del res[-1]
            d = res
            dat = datetime(int(d[2]), self.date_parser[d[1].lower()], int(d[0]), t[0], t[1], 0, tzinfo=self.tz).isoformat("T")
        except:
            dat = datetime(int(self.year), int(self.month), int(self.day), tzinfo=self.tz).isoformat("T")
        item['date'] = dat

        item['title'] = remove_tags(response.xpath('//div[@id="content"]').css('h1.post-heading').extract_first()).strip()

        topic = response.css('ul.post-meta').css('li.cat').css('a').extract_first()
        if topic is not None:
            item['topic'] = remove_tags(topic)

        for p in response.xpath('//div[@class="post-content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
