# -*- coding: utf-8 -*-
import scrapy, re
from periodicoCorreo.items import NoticiasItem

"""
MEDIO:
Periódico Correo, Guanajuato
USO:
scrapy crawl noticias --nolog -s filename=2018-01-29.json -a year=2018 -a month=1 -a day=29
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
G_RE = re.compile(r' ?- ?')
EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "https://periodicocorreo.com.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        lastPage = response.xpath('//*[@class="page-nav td-pb-padding-side"]/a[@class="last"]/text()').extract_first()
        if lastPage is not None and lastPage != '':
            lastPage = int(lastPage)

            for page in range(1, lastPage):
                yield scrapy.Request(url=response.url + "page/" + str(page + 1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.css('div.td-ss-main-content').css('div.item-details').xpath('./h3/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//time[@class="entry-date updated td-module-date"]/@datetime').extract_first()
        item['title'] = remove_tags(response.xpath('//header/h1').extract_first()).strip()

        try:
            topic = response.xpath('//ul[@class="td-category"]/li/a/text()').extract()[-2]
        except:
            topic = response.xpath('//ul[@class="td-category"]/li/a/text()').extract_first()
        item['topic'] = topic

        for p in response.xpath('//div[@class="td-post-content td-pb-padding-side"]/p').extract():
            text += remove_tags(p) + "\n"

        result = LOC_RE.search(text)
        if result:
            m = result.group(0)
            location = G_RE.sub('', m).strip()
            if len(location) <= 35:
                item['location'] = location
                text = text[text.find(m)+len(m):]

        text = EM_RE.sub('', text)
        text = TW_RE.sub('', text)
        text = TW2_RE.sub('', text)
        text = TAG2_RE.sub("\n", text)
        text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
