# -*- coding: utf-8 -*-
import scrapy, re
from periodicoEnfoque.items import NoticiasItem
from datetime import datetime, date, timedelta, tzinfo

"""
MEDIO:
Periódico Enfoque Informativo, Nayarit
USO:
scrapy crawl noticias --nolog -s filename=2018-02-04.json -a year=2018 -a month=2 -a day=4
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')


class UTC(tzinfo):
	"""clase para el 'time zone' (zona horaria)"""

	def utcoffset(self, dt):
		# zona horaria para nayarit: utc-7
		return timedelta(hours=-7)

	def tzname(self, dt):
		# nombre de la zona horaria
		return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        self.year = getattr(self, "year", None)
        self.month = getattr(self, "month", None)
        self.day = getattr(self, "day", None)

        self.baseURL = "https://enfoquenayarit.com/" + self.year + "/" + self.month.zfill(2) + "/" + self.day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        nextPage = response.xpath('//div[@class="post_pagination"]/a[@class="next page-numbers"]/@href').extract_first()
        if nextPage is not None:
            yield scrapy.Request(url=nextPage, callback=self.parse)


    def parse_page(self, response):
        for link in response.xpath('//div[@class="wa-block-news-recent"]').css('h3').css('a::attr(href)').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        try:
            res = response.xpath('//div[@class="entry-meta"]').css('time::attr(datetime)').extract_first()
            d,t = res.split()
            d = map(int, d.split("-"))
            t = map(int, t.split(":"))
            dat = datetime(d[0], d[1], d[2], t[0], t[1], t[2], tzinfo=self.tz).isoformat("T")
        except:
            dat = datetime(int(self.year), int(self.month), int(self.day), tzinfo=self.tz).isoformat("T")
        item['date'] = dat

        item['title'] = remove_tags(response.xpath('//div[@class="wa-post-header"]/div/h1[@class="entry-title"]/a').extract_first()).strip()

        item['topic'] = response.xpath('//ul[@class="wa-ul-breadcumb"]/li/a/text()').extract()[-1]

        author = response.xpath('//span[@itemprop="author"]/text()').extract_first()
        if author is not None:
            item['author'] = author

        for p in response.xpath('//div[@class="entry-content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
