# -*- coding: utf-8 -*-
import scrapy, re
from periodicoVictoria.items import NoticiasItem

"""
MEDIO:
Periódico Victoria, Durango
USO:
scrapy crawl noticias --nolog -s filename=2018-01-28.json -a year=2018 -a month=1 -a day=28
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

LOC_RE = re.compile(r'\A.+?,.+?(\d{1,2}[\s-][a-zA-Z]+[\s-]\d{4})?\s?\.\s?-\s?', re.S)
G_RE = re.compile(r'\s?-\s?')
GT_RE = re.compile(r'&gt;\s?[\w.%+-]+.+')
EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
TW_RE = re.compile(r'Twitter:\s+[\w.%+-]+.')
TW2_RE = re.compile(r'(Twitter:\s+)?(@[\w.%+-]+.)?')
FB_RE = re.compile(r'(Facebook|Blog|Por|Informes|Texto y (F|f)otos):\s+[\w.%+-]+.+')
URL_RE = re.compile(r'\(?(https?:\/\/)?([w{3}.])?[\w%+-]+(\.[a-zA-Z]{2,6}){1,2}[/\w.#$%&+-]*\)?.')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "http://periodicovictoria.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.xpath('//ul[@class="pagination"]/li/a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-1].strip('/')
            pages = int(pagination[pagination.rfind('/') + 1:])

            for page in range(1, pages):
                yield scrapy.Request(url=response.url + "/page/" + str(page + 1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//div[@class="blog"]').css('div.media-body').xpath('./div/h4/a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//meta[@property="article:published_time"]/@content').extract_first()
        item['title'] = remove_tags(response.xpath('//div[@class="article-content"]/header/h1').extract_first()).strip()

        try:
            topic = response.xpath('//ul[@class="breadcrumb"]/li/a/text()').extract()[-2]
        except:
            topic = response.xpath('//ul[@class="breadcrumb"]/li/a/text()').extract_first()
        item['topic'] = topic

        author = response.css('aside.author-bio').xpath('./div/h3/a').extract_first()
        if author is not None and author != '': item['author'] = remove_tags(author)

        for p in response.xpath('//div[@class="post-entry clearfix"]/p').extract():
            p = remove_tags(p)
            p = p.lstrip().replace(u'\u2013', "-")
            p = p.lstrip().replace(u'\u2014', "-")
            p = p.lstrip().replace(u'\u00a0', '')
            result = LOC_RE.match(p)
            if result:
                location = G_RE.sub('', result.group(0)).strip()
                if location.find("Corresponsal") > -1:
                    location = location[location.find("Corresponsal") + len("Corresponsal"):].strip()
                if len(location) <= 30:
                    item['location'] = location
                    p = p.replace(result.group(0), '')
                    text = ''

            text += p + "\n"

        text = GT_RE.sub('', text)
        text = EM_RE.sub('', text)
        text = TW_RE.sub('', text)
        text = TW2_RE.sub('', text)
        text = FB_RE.sub('', text)
        text = URL_RE.sub('', text)
        if text.find("Corresponsal") > -1:
            text = text[text.find("Corresponsal") + len("Corresponsal"):]
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
