# -*- coding: utf-8 -*-
import scrapy, re
from redCrucero.items import NoticiasItem

"""
MEDIO:
Red Crucero, Nuevo León
USO:
scrapy crawl noticias --nolog -s filename=2018-02-04.json -a year=2018 -a month=2 -a day=4
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        baseURL = "http://www.red-crucero.com/news/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        nextPage = response.xpath('//div[@class="nav-previous"]/a/@href').extract_first()
        if nextPage is not None:
            yield scrapy.Request(url=nextPage, callback=self.parse)


    def parse_page(self, response):
        for link in response.xpath('//div[@id="primary"]').css('h2.entry-title').xpath('./a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//span[@class="posted-on"]/a/time/@datetime').extract_first()
        item['title'] = remove_tags(response.xpath('//header[@class="entry-header"]/h1').extract_first()).strip()

        topic = response.xpath('//span[@class="cat-links"]/a/text()').extract_first()
        if topic is not None:
            item['topic'] = topic

        author = response.xpath('//span[@class="author vcard"]/a').extract_first()
        if author is not None:
            author = remove_tags(author).replace("por ", '')
            item['author'] = author

        for p in response.xpath('//div[@class="entry-content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
