# -*- coding: utf-8 -*-
import scrapy, re
from sintesis.items import NoticiasItem

"""
MEDIO:
Síntesis, Puebla
USO:
scrapy crawl noticias --nolog -s filename=2018-02-04.json -a year=2018 -a month=2 -a day=4
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)

# LOC_RE = re.compile(r'\n.+?,? ?.+? ?\. ?- ?')
# G_RE = re.compile(r' ?- ?')
# EM_RE = re.compile(r'((Email|Correo electr.{1,3}nico|Comentarios?):\s)?[\w.-]+@[\w-]+(\.[a-zA-Z]{2,6}){1,2}\s?')
# TW_RE = re.compile(r'M.{1,3}s de la P.{1,3}lvora en Twitter: @[\w.%+-]+.', re.I)
# TW2_RE = re.compile(r'((\| )?Twitter:\s+)?(@[\w.%+-]+.)?', re.I)
# TAG2_RE = re.compile(r'\ntransition_[^\]]+\]')
# TAG3_RE = re.compile(r'\[[^\]]+[\]\n]')

class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        year = getattr(self, "year", None)
        month = getattr(self, "month", None)
        day = getattr(self, "day", None)

        self.baseURL = "https://www.sintesis.mx/" + year + "/" + month.zfill(2) + "/" + day.zfill(2)

        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        yield scrapy.Request(url=response.url, callback=self.parse_page, dont_filter=True)

        pagination = response.css('div.page-nav').xpath('./a/@href').extract()
        if len(pagination) > 0:
            pagination = pagination[-2].strip('/')
            pages = int(pagination[pagination.rfind('/') + 1:])

            for page in range(1, pages):
                yield scrapy.Request(url=self.baseURL + '/page/' + str(page + 1), callback=self.parse_page)


    def parse_page(self, response):
        for link in response.xpath('//div[@class="td-ss-main-content"]').css('h3.entry-title').xpath('./a/@href').extract():
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        "La fecha obtenida ya incluye formato y zona horaria"
        item['date'] = response.xpath('//span[@class="td-post-date"]/time/@datetime').extract_first()
        item['title'] = remove_tags(response.xpath('//header[@class="td-post-title"]/h1').extract_first()).strip()

        topic = response.xpath('//ul[@class="td-category"]/li/a').extract_first()
        if topic is not None:
            item['topic'] = remove_tags(topic)

        author = response.xpath('//div[@class="td-post-author-name"]/a').extract_first()
        if author is not None:
            item['author'] = remove_tags(author)

        for p in response.xpath('//div[@class="td-post-content"]').css('p').extract():
            text += remove_tags(p) + "\n"

        # result = LOC_RE.search(text)
        # if result:
        #     m = result.group(0)
        #     location = G_RE.sub('', m).strip()
        #     if len(location) <= 35:
        #         item['location'] = location
        #         text = text[text.find(m)+len(m):]

        # text = EM_RE.sub('', text)
        # text = TW_RE.sub('', text)
        # text = TW2_RE.sub('', text)
        # text = TAG2_RE.sub("\n", text)
        # text = TAG3_RE.sub('', text)
        item['text'] = text.strip()

        item['url'] = response.url

        yield item
