# -*- coding: utf-8 -*-
import scrapy, re
from solDeMex.items import NoticiasItem
from datetime import datetime, timedelta, tzinfo

"""
MEDIO:
El Sol de México, CDMX
USO:
scrapy crawl noticias --nolog -s filename=2018-01-20.json
"""

TAG_RE = re.compile(r'<[^>]+>')
def remove_tags(text):
    return TAG_RE.sub('', text)


class UTC(tzinfo):
	"""clase para el 'time zone' (zona horaria)"""

	def utcoffset(self, dt):
		# zona horaria para hidalgo (centro de mexico): utc-6
		return timedelta(hours=-6)

	def tzname(self, dt):
		# nombre de la zona horaria
		return 'UTC-6'


class QuotesSpider(scrapy.Spider):
    name = "noticias"

    def start_requests(self):
        self.tz = UTC()
        self.date_parser = {'enero': 1,      'febrero': 2,  'marzo': 3,      'abril': 4,
                            'mayo': 5,       'junio': 6,    'julio': 7,      'agosto': 8,
                            'septiembre': 9, 'octubre': 10, 'noviembre': 11, 'diciembre': 12}

        self.baseURL = 'https://www.elsoldemexico.com.mx/rss.xml'
        yield scrapy.Request(url=self.baseURL, callback=self.parse)


    def parse(self, response):
        for link in response.xpath('//link/text()').extract()[1:]:
            yield scrapy.Request(url=link, callback=self.parse_item)


    def parse_item(self, response):
        item = NoticiasItem()
        text = ''

        d = response.xpath('//p[@class="published-date"]/text()').extract_first()
        d = d.replace('\n','').replace('de ','').split(' ')[-3:]
        item['date'] = datetime(int(d[2]), self.date_parser[d[1].lower()], int(d[0]), tzinfo=self.tz).isoformat('T')
        item['topic'] = response.xpath('//*[@class="pull-left breadcrumb "]/a/text()').extract_first().replace('\n','')
        item['title'] = response.xpath('//h1[@class="title"]/text()').extract_first().replace('\n','')
        for p in response.xpath('//*[@class="content-body clearfix "]/div/p').extract():
            text += remove_tags(p) + '\n'
        item['text'] = text
        item['author'] = response.xpath('//*[@class="affix-start "]/p/text()').extract_first().replace('\n','')
        item['url'] = response.url

        # print item['title']
        yield item

